<?php
/**
 * Created by PhpStorm.
 * User: jderosa
 * Date: 30/10/19
 * Time: 20.04
 */

namespace FiloBlu\Rma\Helper;
use Magento\Framework\Exception\AlreadyExistsException;
use Magento\Sales\Model\Order\Status;
use Magento\Sales\Model\Order\StatusFactory;
use Magento\Sales\Model\ResourceModel\Order\Status as StatusResource;
use Magento\Sales\Model\ResourceModel\Order\StatusFactory as StatusResourceFactory;
use Magento\Sales\Model\Order\Config;
use Magento\Sales\Model\Order;
use Magento\Eav\Model\AttributeRepository;
use Magento\Eav\Model\Config as EavConfig;
use Psr\Log\LoggerInterface;
use Magento\Eav\Setup\EavSetupFactory;

class InstallHelper
{
    const ORDER_STATUS_CHANGESIZE_CODE = 'change_size';

    const ORDER_STATUS_CHANGESIZE_LABEL = 'Change Size Order';

    const REASON_CHANGE_SIZE_LABEL='Change Size';

    const REASON_CHANGE_SIZE_ENTITY_CODE ='rma_item';

    const RMA_RESOLUTION_ATTRIBUTE_CODE = 'resolution';

    const CHANGE_SIZE_RMA_ATTRIBUTE_CODE = 'change-size';

    const RESOLUTION_CHANGE_SIZE_OPTION_CODE = 'change_size';

    const REASON_OTHER_RMA_ATTRIBUTE_CODE = 'reason_other';

    /**
     * @var StatusFactory
     */
    protected $statusFactory;

    /**
     * @var StatusResourceFactory
     */
    protected $statusResourceFactory;

    /**
     * @var Config
     */
    protected $orderStatus;

    /**
     * @var Order
     */
    protected $order;

    /**
     * @var AttributeRepository
     */
    protected $attributeRepository;

    /**
     * @var EavConfig
     */
    protected $eavConfig;

    /**
     * @var LoggerInterface
     */
    protected $logger;

    /**
     * @var EavSetupFactory
     */
    protected $eavSetupFactory;


    /**
     * InstallHelper constructor.
     * @param StatusFactory $statusFactory
     * @param StatusResourceFactory $statusResourceFactory
     * @param Config $orderStatus
     * @param Order $order
     * @param AttributeRepository $attributeRepository
     * @param EavConfig $eavConfig
     * @param LoggerInterface $logger
     * @param EavSetupFactory $eavSetupFactory
     */
    public function __construct(StatusFactory $statusFactory,
                                StatusResourceFactory $statusResourceFactory,
                                Config $orderStatus,
                                Order $order,
                                AttributeRepository $attributeRepository,
                                EavConfig $eavConfig,
                                LoggerInterface $logger,
                                EavSetupFactory $eavSetupFactory)
    {

        $this->statusFactory = $statusFactory;
        $this->statusResourceFactory = $statusResourceFactory;
        $this->orderStatus = $orderStatus;
        $this->order = $order;
        $this->attributeRepository = $attributeRepository;
        $this->eavConfig = $eavConfig;
        $this->logger = $logger;
        $this->eavSetupFactory = $eavSetupFactory;
    }


    /**
     * @return bool
     */
    protected function checkIfStateAndStatusExists()
    {
        $exists = false;
        $allStatus = $this->orderStatus->getStatuses();
        if(in_array(self::ORDER_STATUS_CHANGESIZE_LABEL,$allStatus))
        {
            $exists = true;
        }
        return $exists;
    }


    /**
     * Create new custom order status and assign it to the new custom order state
     *
     * @return void
     *
     * @throws Exception
     */
    public function addNewOrderStatus($setup)
    {
        try
        {
            if(!$this->checkIfStateAndStatusExists())
            {
                /** @var Status $status */
                $status = $this->statusFactory->create(['setup'=>$setup]);
                $status->setData([
                    'status' => self::ORDER_STATUS_CHANGESIZE_CODE,
                    'label' => self::ORDER_STATUS_CHANGESIZE_LABEL
                ]);
                /** @var StatusResource $statusResource */
                $statusResource = $this->statusResourceFactory->create(['setup'=>$setup]);
                $statusResource->save($status);
                $status->assignState($this->order::STATE_NEW, false, true);
            }
            else
            {
                $this->logger->debug("Skipping creation of new order status : already exists!");
            }
        }
        catch (\Exception $exception)
        {
            $this->logger->critical($exception->getMessage());
        }
    }


    public function addNewReasonOption($setup)
    {
        try
        {
            $eavSetup = $this->eavSetupFactory->create(['setup'=>$setup]);
            $resolution = $this->attributeRepository->get(self::REASON_CHANGE_SIZE_ENTITY_CODE,self::RMA_RESOLUTION_ATTRIBUTE_CODE);
            $resolutionOptions = $resolution->getOptions();
            $exists = false;
            foreach ($resolutionOptions as $option)
            {
                if($option->getLabel() == self::REASON_CHANGE_SIZE_LABEL)
                {
                    $exists = true;
                }
            }
            if(!$exists)
            {
                $resolutionAttributeId = $resolution->getAttributeId();
                $options = [
                    'values'=>[
                        self::RESOLUTION_CHANGE_SIZE_OPTION_CODE =>self::REASON_CHANGE_SIZE_LABEL
                    ],
                    'attribute_id'=>$resolutionAttributeId
                ];

                $eavSetup->addAttributeOption($options);
            }
        }
        catch (\Exception $e)
        {
            $this->logger->critical($e->getMessage());
        }
    }


    public function addNewRmaItem($setup)
    {
        try{
            $eavSetup = $this->eavSetupFactory->create(['setup'=>$setup]);
            $rmaItemEntityTypeId = $this->eavConfig->getEntityType(self::REASON_CHANGE_SIZE_ENTITY_CODE)->getEntityTypeId();
            $eavSetup->addAttribute($rmaItemEntityTypeId,self::CHANGE_SIZE_RMA_ATTRIBUTE_CODE,[
                'label' => self::REASON_CHANGE_SIZE_LABEL,
                'input' => 'select',
                'global' => \Magento\Eav\Model\Entity\Attribute\ScopedAttributeInterface::SCOPE_GLOBAL,
                'visible' => true,
                'required' => false,
                'user_defined' => true,
                'visible_on_front' => true,
                'backend_type' => 'int',
                'source' => 'Magento\Eav\Model\Entity\Attribute\Source\Table'
            ]);

            $changeSizeRmaAttribute = $this->attributeRepository->get(self::REASON_CHANGE_SIZE_ENTITY_CODE,self::CHANGE_SIZE_RMA_ATTRIBUTE_CODE);

            $attribute = $this->eavConfig->getAttribute(self::REASON_CHANGE_SIZE_ENTITY_CODE,self::CHANGE_SIZE_RMA_ATTRIBUTE_CODE);
            $attribute->setData('used_in_forms',["default"]);
            $attribute->save();

        }catch (\Exception $e){
            $this->logger->critical($e->getMessage());
        }
    }

    public function hideReasonOther($setup){
        $eavSetup = $this->eavSetupFactory->create(['setup'=>$setup]);
        $rmaItemEntityTypeId = $this->eavConfig->getEntityType(self::REASON_CHANGE_SIZE_ENTITY_CODE)->getEntityTypeId();
        $attributeId = $this->attributeRepository->get(self::REASON_CHANGE_SIZE_ENTITY_CODE,self::REASON_OTHER_RMA_ATTRIBUTE_CODE)->getAttributeId();
        $eavSetup->updateAttribute($rmaItemEntityTypeId,$attributeId,[
            'is_visible'=>0
        ]);
    }


}